# Если версия обновления на сайте ФИАС не содержит ссылки на файл с данными,
# то процесс обновления завершится ошибкой. Если в течение указанного срока с даты
# создания дампа версии ссылки не появилось, то эта версия будет пропущена
# и процесс обновления продолжится
VERSION_WITHOUT_URL_TIMEOUT_HOURS = 7 * 24  # неделя
