# Generated by Django 3.2.6 on 2022-02-17 05:55

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('m3_gar', '0006_auto_20211105_0937'),
    ]

    operations = [
        migrations.AddField(
            model_name='version',
            name='processed',
            field=models.BooleanField(default=False, verbose_name='Обработано'),
        ),
    ]
