# Generated by Django 3.2.4 on 2025-04-14 10:18

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('m3_gar', '0022_auto_20250331_0342'),
    ]

    operations = [
        migrations.AlterField(
            model_name='operationtypes',
            name='name',
            field=models.CharField(max_length=300, verbose_name='Наименование'),
        ),
    ]
